/**
 * Copyright (c) 1996-2023, SR Research Ltd., All Rights Reserved
 *
 * For use by SR Research licencees only. Redistribution and use in source
 * and binary forms, with or without modification, are NOT permitted.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 *
 * Neither name of SR Research Ltd nor the name of contributors may be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __SR_RESEARCH_FOLDER_SUPPORT_H__
#define __SR_RESEARCH_FOLDER_SUPPORT_H__

#ifdef __cplusplus     /* For C++ compilation */
extern "C" {
#endif

/*
 * Returns the path to the output folder.
 * For macos and linux, default output folder is ~/Documents/<exe name>_<date string>
 * For Windows default output folder behaviour is as follows.
 *take the first pass case. ie. if item #1 passes, take it otherwise move on to the next item.
 *1. if set_output_folder is called use that path.
 *2. Can the current directory writeable. If yes, use the current directory as output_folder. (This will handle the 32bit application case)
 *3. Can we write to the PublicDocuments folder? if yes, use the public documents folder
 *4. Can we write to the Documents folder? if yes use the documents folder
 *5. If we reach here that means we cannot write anywhere. We can attempt the desktop, but I guess, if documents folder is not writeable, then chances are we cannot write anywhere.
 
 */
const char * get_output_folder(void);

/*
 Allow one to set the output folder.  If output folder needs to be created, call this before calling initialize_sdl_util.
 Otherwise, this only sets teh output folder.
 */
int set_output_folder(const char *path);

/*
 * Convenient function to copy a resource to output folder.
 */
int copy_resource_to_output_folder(const char *resource);

	
/*
 * Concenient function to show the output folder.
 *
 */
int open_output_folder(void);
/*
 *initializes the folder util optionally creates the output folder.
 *returns non zero on success. 0 if failed.
 * see get_output_folder() for more detail about the get_output_folder behaviour.
 * exe_path is the path of the executable. eg. argv[0] of the main().
 * output_folder if None NULL, use this as the default output folder.
 */
int initialize_folder(const char *exe_path, const char *output_folder, int create_output_folder);

#ifdef __cplusplus     /* For C++ compilation */
}
#endif

#endif